@@ -787,27 +787,11 @@ static int au0828_i2s_init(struct au0828_dev *dev)
 
 /*
  * Auvitek au0828 analog stream enable
+ * Please set interface0 to AS5 before enable the stream
  */
 static int au0828_analog_stream_enable(struct au0828_dev *d)
 {
-	struct usb_interface *iface;
-	int ret;
-
 	dprintk(1, "au0828_analog_stream_enable called\n");
-
-	iface = usb_ifnum_to_if(d->usbdev, 0);
-	if (iface && iface->cur_altsetting->desc.bAlternateSetting != 5) {
-		dprintk(1, "Changing intf#0 to alt 5\n");
-		/* set au0828 interface0 to AS5 here again */
-		ret = usb_set_interface(d->usbdev, 0, 5);
-		if (ret < 0) {
-			printk(KERN_INFO "Au0828 can't set alt setting to 5!\n");
-			return -EBUSY;
-		}
-	}
-
-	/* FIXME: size should be calculated using d->width, d->height */
-
 	au0828_writereg(d, AU0828_SENSORCTRL_VBI_103, 0x00);
 	au0828_writereg(d, 0x106, 0x00);
 	/* set x position */
@@ -1018,6 +1002,15 @@ static int au0828_v4l2_open(struct file *filp)
 		return -ERESTARTSYS;
 	}
 	if (dev->users == 0) {
+		/* set au0828 interface0 to AS5 here again */
+		ret = usb_set_interface(dev->usbdev, 0, 5);
+		if (ret < 0) {
+			mutex_unlock(&dev->lock);
+			printk(KERN_INFO "Au0828 can't set alternate to 5!\n");
+			kfree(fh);
+			return -EBUSY;
+		}
+
 		au0828_analog_stream_enable(dev);
 		au0828_analog_stream_reset(dev);
 
@@ -1116,7 +1109,7 @@ static void au0828_init_tuner(struct au0828_dev *dev)
 	/* If we've never sent the standard in tuner core, do so now.
 	   We don't do this at device probe because we don't want to
 	   incur the cost of a firmware load */
-	v4l2_device_call_all(&dev->v4l2_dev, 0, core, s_std, dev->std);
+	v4l2_device_call_all(&dev->v4l2_dev, 0, video, s_std, dev->std);
 	v4l2_device_call_all(&dev->v4l2_dev, 0, tuner, s_frequency, &f);
 	i2c_gate_ctrl(dev, 0);
 }
@@ -1259,6 +1252,13 @@ static int au0828_set_format(struct au0828_dev *dev, unsigned int cmd,
 		}
 	}
 
+	/* set au0828 interface0 to AS5 here again */
+	ret = usb_set_interface(dev->usbdev, 0, 5);
+	if (ret < 0) {
+		printk(KERN_INFO "Au0828 can't set alt setting to 5!\n");
+		return -EBUSY;
+	}
+
 	au0828_analog_stream_enable(dev);
 
 	return 0;
@@ -1368,7 +1368,7 @@ static int vidioc_s_std(struct file *file, void *priv, v4l2_std_id norm)
 	   have to make the au0828 bridge adjust the size of its capture
 	   buffer, which is currently hardcoded at 720x480 */
 
-	v4l2_device_call_all(&dev->v4l2_dev, 0, core, s_std, norm);
+	v4l2_device_call_all(&dev->v4l2_dev, 0, video, s_std, norm);
 
 	i2c_gate_ctrl(dev, 0);
 
